/***********************************************************
constants
***********************************************************/

// reference to the interface defined in cmguiIIsolineFilter.idl
const cmguiIIsolineFilter = Components.interfaces.cmguiIIsolineFilter;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// references to other required interfaces
// const nsIDOMWindow = Components.interfaces.nsIDOMWindow;

//  uniquely identifying our component
const CLASS_ID = Components.ID("{165FCF18-E7D8-11DB-95B5-AF2F56D89593}");

// description
const CLASS_NAME = "Cmgui isoline filter XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@bioeng.auckland.ac.nz/isolinefilter;1";

/***********************************************************
class definition
***********************************************************/


//class constructor
function IsolineFilter() {
};



// class definition
IsolineFilter.prototype = {

  // members
	parent: null,
	commandData: null,
	isovalue: null,
	density: 1,

	inputFieldName: null,
  inputField: null,
  callback: null,

	gElementName: null,
	sceneViewer: null,

// define the function methods we want to expose in our interface
// outputFieldName not required, just present to match other idl init functions
  init: function(parentWindow, commandData, inputFieldName, outputFieldName, callback) {

		this.parent = parentWindow;
		this.commandData = commandData;         // command data object for accessing cmgui commands    
		this.inputFieldName = inputFieldName;	
		this.callback = callback;
		
		// find the input field, note it is assumed to exist
	  this.inputField = commandData.findFieldByName(inputFieldName);

		// create an output field
		// even though this component creates nodes rather than filtering the image we still
		// need to provide an output image
		commandData.executeCommand("gfx def field " + outputFieldName + " scale field " + inputFieldName + " scale_factors 1;");  

		
		// set some defaults which will be overwritten if the field exists
		this.isovalue = 0.15;
		this.density = 1;

		// upping the discretizaton screws with the digitise_plane location in the view_scene scene,
		// the wrong coordinates are picked up so we need to specificly reset the default coordinates
		commandData.executeCommand("gfx modify g_element digitise_plane general element_discre \"128*128\";");
		
		//		commandData.executeCommand("gfx modify g_element digitise_plane general element_discre \"128*128*128\" scene default default_coordinate ndc_inv;");
		
		// clear out ngroup if it exists
		this.clear_ngroup();
		this.generate_points();


  },


	// generates data points along isolines
	generate_points: function() {
		
		var commandData = this.commandData;

		// create/modify the density field that governs the density of points created along the isoline
		commandData.executeCommand("gfx define field point_density constant " + this.density + ";");
	 
		// create isolines
		commandData.executeCommand("gfx modify g_element digitise_plane iso_surfaces as isolines material red coordinate sliceTextureCoordinates iso_scalar input_image iso_value " + this.isovalue + " data point_density visible scene default;");	
		
		// convert them to data points
		commandData.executeCommand("gfx convert graphics scene default.digitise_plane.isolines render_surface_node_cloud region isoline_points coordinate coordinates;");
		
		// draw the data points
		commandData.executeCommand("gfx modify g_element isoline_points node mat blue glyph sphere size 4 scene default;");
		commandData.executeCommand("gfx modify g_element isoline_points node mat blue glyph sphere size 4 scene view_scene;");
	},
	
	
	clear_ngroup: function() {
		
		var commandData = this.commandData;

		// if isoline_points exist clear them out
		commandData.executeCommand("gfx destroy node group isoline_points");
		//		commandData.executeCommand("gfx modify ngroup isoline_points remove all");
		//		commandData.executeCommand("gfx destroy ngroup isoline_points");
		
	},
	
	
	// call this function from the dialog to take care of updating the filter and running the callback
  apply_filter: function() {
		
		this.clear_ngroup();
		this.generate_points();

		this.callback.run();
		
  },
	

	create_dialog: function() {

		var parent = this.parent;
		var params={isolineFilter:this};


		try {
						
      dialogReference = parent.openDialog("chrome://zinc/content/filters/isolineFilterDialog.xul",
																					"isoline-filter-dialog",
																					"chrome",params);
			
			// store reference to dialog and associated field name
			parent.isolineFilterDialogReference = dialogReference;

		}catch(ex) {
			parent.alert("Error creating dialog to edit discrete gaussian filter: " + ex);
		}

	},
	

  open_dialog: function(gElementName, sceneViewer) {

		this.gElementName = gElementName;
		this.sceneViewer = sceneViewer;

		// window parent
		var parent = this.parent;
		var dialogReference = parent.isolineFilterDialogReference;

		// determine if a dialog already exists
		if (dialogReference == null || dialogReference.closed) {
			
			// no dialog so create one
			this.create_dialog();
			
		} else { 
			
				// correct dialog so bring it to the front
				dialogReference.focus();

				// if the dialog already exists we also need some way of extracting out the previous
				// settings that were applied.  Normally these are accessible by using idl routines
				// to pull the values out of an existing field.  In this case we have no field
				// where should those values go?
		}
		
  },
	
    

  // the QueryInterface function throws an exception
  // if the cmguiIGeneralThresholdFilter interface is not implemented
  // otherwise it returns a reference to the GeneralThresholdFilter object
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(cmguiIIsolineFilter) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myIsolineFilter = Components.classes["@bioeng.auckland.ac.nz/helloworld;1"].
                          createInstance(Components.interfaces.cmguiIIsolineFilter);

***********************************************************/
var IsolineFilterFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new IsolineFilter()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var IsolineFilterModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return IsolineFilterFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return IsolineFilterModule; }
